from RDATKit import rdatkit
import draw_rna.draw as d
import sys,os
import argparse
import numpy as np

def draw_rdat_construct(rdat_file, construct_id):


    rdat_identifier=os.path.basename(rdat_file).replace('.rdat','')
    
    rdat = rdatkit.RDATFile()

    # load RDAT file
    rdat.load(open(rdat_file))

    seqpos = [x-1 for x in rdat.constructs['EteRNA Cloud Lab'].seqpos]
    seq=rdat.constructs['EteRNA Cloud Lab'].data[construct_id].annotations['sequence'][0]
    secstruct = rdat.constructs['EteRNA Cloud Lab'].data[construct_id].annotations['structure'][0]

    assert len(seq) == len(secstruct)

    reactivities = rdat.constructs['EteRNA Cloud Lab'].data[construct_id].values

    reactivities.extend([0]*(len(seq) - len(reactivities)))
    name = '%s_%d' % (os.path.basename(rdat_identifier), construct_id)
    print('Drawing construct:')
    print(seq)
    print(secstruct)
    print(name)
    d.draw_rna(seq, secstruct, np.array(reactivities), filename=name, ext_color_file=True, cmap_name='gist_heat_r', chemical_mapping_mode=True)

def main():
    p = argparse.ArgumentParser()
    p.add_argument('inputfile')
    p.add_argument('construct_id')
    p.add_argument('--png', action='store_true',default=False)
    p.add_argument('--movie_mode', action='store_true')
    p.add_argument('--large_mode',action='store_true')

    args = p.parse_args()
    print(args)

    rdat_identifier=os.path.basename(args.inputfile).replace('.rdat','')
    construct_id = int(args.construct_id)
    draw_rdat_construct(args.inputfile, construct_id)

    if args.png:
        name = '%s_%d' % (rdat_identifier, construct_id)
        if 'INKSCAPEDIR' not in os.environ:
            print('Please set INKSCAPEDIR environmental variable with path to Inkscape app.')
            return
        inkscape = '%s/Contents/Resources/bin/inkscape' % os.environ['INKSCAPEDIR']
        if not os.path.isfile(inkscape):
            print('Inkscape not found. Please update INKSCAPEDIR environmental variable with path to Inkscape app.')
            return
        os.system('%s --export-png $(pwd)/%s.png --export-background-opacity 1 $(pwd)/%s.svg' % (inkscape, name, name))
        os.system('rm $(pwd)/%s.svg' % (name))

if __name__=='__main__':
    main()



import draw_rna.svg as svg
import re, random, math
import numpy as np

class RNATreeNode:

    def __init__(self):
        self.children_ = []
        self.is_pair_ = False
        self.index_a_ = -1
        self.index_b_ = -1
        self.x_ = 0
        self.y_ = 0
        self.go_x_ = 0
        self.go_y_ = 0

def get_pairmap_from_secstruct(secstruct):
    """
    generates dictionary containing pair mappings

    args:
    secstruct contains secondary structure string

    returns:
    dictionary with pair mappings
    """
    pair_stack = []
    end_stack = []
    pairs_array = []
    i_range = list(range(0,len(secstruct)))

    # initialize all values to -1, meaning no pair
    for ii in i_range:
        pairs_array.append(-1)

    # assign pairs based on secstruct
    for ii in i_range:
        if(secstruct[ii] == "("):
            pair_stack.append(ii)
        elif(secstruct[ii] == ")"):
            if not pair_stack:
                end_stack.append(ii)
            else:
                index = pair_stack.pop()
                pairs_array[index] = ii
                pairs_array[ii] = index
    if len(pair_stack) == len(end_stack):
        n = len(pair_stack)
        for ii in range(n):
            pairs_array[pair_stack[ii]] = end_stack[-ii]
            pairs_array[end_stack[-ii]] = pair_stack[ii]
    else:
         print("ERROR: pairing incorrect %s" % secstruct)

    return pairs_array

def get_pk_pairmap_from_secstruct(s, allow_pseudoknots=False):
    lefts = ["[", "{", "<"]
    rights = ["]", "}", ">"]
    lower_alphabet = [chr(lower) for lower in range(97, 123)]
    upper_alphabet = [chr(upper) for upper in range(65, 91)]
    lefts.extend(lower_alphabet)
    rights.extend(upper_alphabet)

    m = [-1 for i in range(len(s))]
    for left, right in zip(lefts, rights):
        bp1 = []
        bp2 = []
        for i, char in enumerate(s):
            if char == left:
                bp1.append(i)
            elif char == right:
                bp2.append(i)

        for i in list(reversed(bp1)):
            for j in bp2:
                if j > i:
                    m[i] = j
                    m[j] = i

                    bp2.remove(j)
                    break

    return m

def add_nodes_recursive(bi_pairs, rootnode, start_index, end_index):

    if(start_index > end_index) :
        print("Error occured while drawing RNA %d %d" % (start_index, end_index))
        raise RuntimeError("Error occured while drawing RNA %d %d" % (start_index, end_index))

    if(bi_pairs[start_index] == end_index) :

        newnode = RNATreeNode()
        newnode.is_pair_ = True
        newnode.index_a_ = start_index
        newnode.index_b_ = end_index

        add_nodes_recursive(bi_pairs, newnode, start_index+1, end_index-1)

    else :
        newnode = RNATreeNode()
        jj = start_index
        while jj <= end_index:
            if(bi_pairs[jj] > jj) :
                add_nodes_recursive(bi_pairs,newnode, jj, bi_pairs[jj])
                jj = bi_pairs[jj] + 1
            else :
                newsubnode = RNATreeNode()
                newsubnode.is_pair_ = False
                newsubnode.index_a_ = jj
                newnode.children_.append(newsubnode)
                jj += 1

    rootnode.children_.append(newnode)

def setup_coords_recursive(rootnode, parentnode, start_x, start_y, go_x, go_y, NODE_R, PRIMARY_SPACE, PAIR_SPACE, external_multiplier, external_offset, flipped):

    cross_x = -go_y
    cross_y = go_x

    children_width = len(rootnode.children_) * NODE_R * 2

    #print('children_width', children_width)

    rootnode.go_x_ = go_x
    rootnode.go_y_ = go_y

    if(len(rootnode.children_) == 1):
        rootnode.x_ = start_x
        rootnode.y_ = start_y

        if(rootnode.children_[0].is_pair_):
            setup_coords_recursive(rootnode.children_[0], rootnode, start_x + go_x * PRIMARY_SPACE, start_y + (-1 if flipped else 1) * go_y * PRIMARY_SPACE, go_x, go_y, NODE_R, PRIMARY_SPACE, PAIR_SPACE, external_multiplier, external_offset, flipped)
        elif(rootnode.children_[0].is_pair_ == False and rootnode.children_[0].index_a_ < 0):
            setup_coords_recursive(rootnode.children_[0], rootnode, start_x, start_y, go_x, go_y, NODE_R, PRIMARY_SPACE, PAIR_SPACE, external_multiplier, external_offset, flipped)
        else:
            setup_coords_recursive(rootnode.children_[0], rootnode, start_x + go_x * PRIMARY_SPACE, start_y + (-1 if flipped else 1) * go_y * PRIMARY_SPACE, go_x, go_y, NODE_R, PRIMARY_SPACE, PAIR_SPACE, external_multiplier, external_offset, flipped)

    elif(len(rootnode.children_) > 1) :

        npairs = 0
        for ii in range(0, len(rootnode.children_)):
            if(rootnode.children_[ii].is_pair_) :
                npairs+=1

        circle_length = (len(rootnode.children_) + 1) * PRIMARY_SPACE + (npairs + 1) * PAIR_SPACE
        circle_radius = circle_length / (2 * math.pi)

        length_walker = PAIR_SPACE / 2.0

        if (parentnode == None) :
            rootnode.x_ = go_x * circle_radius
            rootnode.y_ = go_y * circle_radius
            circle_radius *= external_multiplier
        else :
            rootnode.x_ = parentnode.x_ + go_x * circle_radius
            rootnode.y_ = parentnode.y_ + (-1 if flipped else 1) * go_y * circle_radius

        for ii in range(0,len(rootnode.children_)):

            if (parentnode == None):
                length_walker += PRIMARY_SPACE

                if(rootnode.children_[ii].is_pair_) :
                    length_walker += PAIR_SPACE / 2.0

                rad_angle = length_walker/circle_length * 2 * math.pi / external_multiplier + (1 if flipped else -1) * math.pi / 2.0 + external_offset
                child_x = rootnode.x_ + math.cos(rad_angle) * cross_x * circle_radius + math.sin(rad_angle) * go_x * circle_radius
                child_y = rootnode.y_ + (-1 if flipped else 1) * math.cos(rad_angle) * cross_y * circle_radius + (-1 if flipped else 1) * math.sin(rad_angle) * go_y * circle_radius

                child_go_x = child_x - rootnode.x_
                child_go_y = child_y - rootnode.y_
                child_go_len = math.sqrt(child_go_x * child_go_x + child_go_y * child_go_y)

                setup_coords_recursive(rootnode.children_[ii], rootnode, child_x, child_y, child_go_x / child_go_len, (-1 if flipped else 1) * child_go_y / child_go_len, NODE_R, PRIMARY_SPACE, PAIR_SPACE, external_multiplier, external_offset, flipped)

                if(rootnode.children_[ii].is_pair_) :
                    length_walker += PAIR_SPACE / 2.0

            else:

                length_walker += PRIMARY_SPACE

                if(rootnode.children_[ii].is_pair_) :
                    length_walker += PAIR_SPACE / 2.0

                rad_angle = length_walker/circle_length * 2 * math.pi - math.pi / 2.0 + external_offset
                child_x = rootnode.x_ + math.cos(rad_angle) * cross_x * circle_radius + math.sin(rad_angle) * go_x * circle_radius
                child_y = rootnode.y_ + (-1 if flipped else 1) * math.cos(rad_angle) * cross_y * circle_radius + (-1 if flipped else 1) * math.sin(rad_angle) * go_y * circle_radius

                child_go_x = child_x - rootnode.x_
                child_go_y = child_y - rootnode.y_
                child_go_len = math.sqrt(child_go_x * child_go_x + child_go_y * child_go_y)

                setup_coords_recursive(rootnode.children_[ii], rootnode, child_x, child_y, child_go_x / child_go_len, (-1 if flipped else 1) * child_go_y / child_go_len, NODE_R, PRIMARY_SPACE, PAIR_SPACE, external_multiplier, external_offset, flipped)

                if(rootnode.children_[ii].is_pair_) :
                    length_walker += PAIR_SPACE / 2.0

    else :
        rootnode.x_ = start_x
        rootnode.y_ = start_y

def get_coords_recursive(rootnode, xarray, yarray, PRIMARY_SPACE, PAIR_SPACE, flipped):
    if(rootnode.is_pair_) :
        cross_x = -rootnode.go_y_
        cross_y = rootnode.go_x_

        xarray[rootnode.index_a_] = rootnode.x_ + cross_x * PAIR_SPACE/2.0
        xarray[rootnode.index_b_] = rootnode.x_ - cross_x * PAIR_SPACE/2.0

        yarray[rootnode.index_a_] = rootnode.y_ + (-1 if flipped else 1) * cross_y * PAIR_SPACE/2.0
        yarray[rootnode.index_b_] = rootnode.y_ + (1 if flipped else -1) * cross_y * PAIR_SPACE/2.0
    elif(rootnode.index_a_ >= 0) :
        xarray[rootnode.index_a_] = rootnode.x_
        yarray[rootnode.index_a_] = rootnode.y_

    for ii in range(0, len(rootnode.children_)):
        get_coords_recursive(rootnode.children_[ii], xarray, yarray, PRIMARY_SPACE, PAIR_SPACE, flipped)



class RNARenderer:

    def __init__(self):
        self.root_ = None
        self.xarray_ = None
        self.yarray_ = None
        self.size_ = None

    def setup_tree(self, secstruct, NODE_R,PRIMARY_SPACE, PAIR_SPACE, external_multiplier, external_offset, flipped):

        dangling_start = 0
        dangling_end = 0
        bi_pairs = get_pairmap_from_secstruct(secstruct)

        self.NODE_R = NODE_R
        self.root_ = None

        for ii in range(0,len(bi_pairs)):
            if bi_pairs[ii] < 0:
                dangling_start+=1
            else:
                break

        for ii in (len(bi_pairs)-1,-1, -1) :
            if(bi_pairs[ii] < 0):
                dangling_end+=1
            else:
                break

        self.root_ = RNATreeNode()

        #for jj in range(0,len(bi_pairs)):
        jj = 0
        while (jj < len(bi_pairs)):
            if (bi_pairs[jj] > jj) :
                add_nodes_recursive(bi_pairs,self.root_, jj, bi_pairs[jj])
                jj = bi_pairs[jj] + 1
            else:
                newsubnode = RNATreeNode()
                newsubnode.is_pair_ = False
                newsubnode.index_a_ = jj
                self.root_.children_.append(newsubnode)
                jj += 1
        xarray = []
        yarray = []

        for ii in range(0,len(secstruct)):
            xarray.append(0.0)
            yarray.append(0.0)

        self.setup_coords(NODE_R,PRIMARY_SPACE,PAIR_SPACE, external_multiplier, external_offset, flipped)
        self.get_coords(xarray,yarray,PRIMARY_SPACE,PAIR_SPACE, flipped)

        min_x = xarray[0] - NODE_R
        min_y = yarray[0] - NODE_R
        max_x = xarray[0] + NODE_R
        max_y = xarray[0] + NODE_R

        for x in xarray:
            if x - NODE_R < min_x:
                min_x = x - NODE_R
            if x + NODE_R > max_x:
                max_x = x + NODE_R

        for y in yarray:
            if y - NODE_R < min_y:
                min_y = y - NODE_R
            if y + NODE_R > max_y:
                max_y = y + NODE_R

        for ii in range(0,len(xarray)):
            xarray[ii] -= min_x
            yarray[ii] -= min_y

        self.size_ = [max_x - min_x, max_y - min_y]
        self.xarray_ = xarray
        self.yarray_ = yarray

    def get_size(self):
        return self.size_

    def draw(self, svgobj, offset_x, offset_y, colors, pairs, pk_pairs, sequence, render_in_letter, external_offset, flipped, line=False, svg_mode=True, alpha=None, direction_labels=True):
        if alpha is None:
            alpha = np.ones(len(self.xarray_))

        if self.xarray_ != None:

            if line:
                for ii in range(len(self.xarray_)-1):
                    if colors == None:
                        svgobj.line(self.xarray_[ii], self.yarray_[ii], self.xarray_[ii+1], self.yarray_[ii+1],
                                    'black')
                    else:
                        svgobj.line(self.xarray_[ii], self.yarray_[ii], self.xarray_[ii+1], self.yarray_[ii+1],
                                    colors[ii])
            else:
                if pairs:
                    for pair in pairs:
                        svgobj.line(offset_x + self.xarray_[pair['from']], offset_y + self.yarray_[pair['from']],
                         offset_x + self.xarray_[pair['to']], offset_y + self.yarray_[pair['to']],
                          pair['color'], self.NODE_R, alpha=min([alpha[pair['from']],alpha[pair['to']]]))
                
                if pk_pairs:
                    for pair in pk_pairs:
                        svgobj.line(offset_x + self.xarray_[pair['from']], offset_y + self.yarray_[pair['from']],
                         offset_x + self.xarray_[pair['to']], offset_y + self.yarray_[pair['to']],
                          pair['color'], self.NODE_R / 4, alpha=min([alpha[pair['from']],alpha[pair['to']]]))

                for ii in range(0,len(self.xarray_)):
                    if colors == None:
                        svgobj.circle(self.xarray_[ii] + offset_x, self.yarray_[ii] + offset_y, self.NODE_R, "#000000", "#000000", alpha[ii])
                    else:
                        svgobj.circle(self.xarray_[ii] + offset_x, self.yarray_[ii] + offset_y, self.NODE_R, colors[ii], colors[ii], alpha[ii])

                if sequence and render_in_letter:

                    # write 5' 3' markers
                    if direction_labels:
                        svgobj.text(self.xarray_[0] + offset_x + (1 if flipped else -1) * math.sin(external_offset)*2.5*self.NODE_R,
                        self.yarray_[0] + offset_y + (1 if flipped else -1) * math.cos(external_offset)* 2.5*self.NODE_R, self.NODE_R * 1.5, "#000000", "center", "5'", 1, 'top')

                        svgobj.text(self.xarray_[-1] + offset_x + (1 if flipped else -1) * math.sin(external_offset)*2.5*self.NODE_R,
                        self.yarray_[-1] + offset_y + (1 if flipped else -1) * math.cos(external_offset)* 2.5*self.NODE_R, self.NODE_R * 1.5, "#000000", "center", "3'", 1, 'top')

                    for ii in range(0,len(self.xarray_)):
                        text_size = self.NODE_R * 1.5
                        if colors[ii] == [0,0,0]:
                            color = "#FFFFFF"
                        else:
                            color = "#000000"

                        svgobj.text(self.xarray_[ii] + offset_x, self.yarray_[ii] + offset_y, text_size, color, "center", sequence[ii], alpha[ii])

    def get_coords(self, xarray, yarray, PRIMARY_SPACE, PAIR_SPACE, flipped):

        if(self.root_ != None) :
            get_coords_recursive(self.root_, xarray, yarray, PRIMARY_SPACE, PAIR_SPACE, flipped)
        else :
            for ii in range(0,len(xarray)):
                xarray[ii] = 0
                yarray[ii] = ii * PRIMARY_SPACE

    def setup_coords(self, NODE_R, PRIMARY_SPACE, PAIR_SPACE, external_multiplier, external_offset, flipped):
        if self.root_ != None:
            setup_coords_recursive(self.root_, None, 0, 0, 0, 1, NODE_R, PRIMARY_SPACE, PAIR_SPACE, external_multiplier, external_offset, flipped)






import draw_rna.svg as svg
import draw_rna.mpl as mpl
import draw_rna.render_rna as render_rna
import draw_rna.inv_utils as inv_utils
import numpy as np
import argparse
import re
from io import BytesIO
from matplotlib import cm
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors


NODE_R = 10
PRIMARY_SPACE = 20
PAIR_SPACE = 20

TEXT_SIZE = 50

RENDER_IN_LETTERS = True

COLORS = {#"r": [255, 0, 0],
          "r": [255, 102, 102],
          "g": [113, 188, 120],
          "b": [51, 153, 255],
          "p": [255, 210, 200],
          "k": [1, 0, 0],
          "y": [255, 211, 0],
          "c": [0, 255, 255],
          "m": [255, 0, 255],
          "w": [255, 255, 255],
          "e": [100, 100, 100],
          "f": [200, 200, 200],
          "o": [231, 115, 0],
          "i": [51, 204, 204],
          "h": [51, 153, 255],
          "u": [138, 43, 226]}
          #"h": [46, 184, 46]}

def draw_rna(sequence, secstruct, color_list, filename="secstruct", line=False,
    cmap_name='viridis', rotation=0, alpha=None,
    ext_color_file=False, chemical_mapping_mode=False, 
    large_mode=False, movie_mode=False, svg_mode=False, vmin=None, vmax=None, ax=None,
    flipped=False, padding=None, square=True, draw_pairs=True, draw_pseudoknots=True,
    direction_labels=True, show_colorbar='auto', colorbar_label=None
):

    CELL_PADDING = padding

    if large_mode or movie_mode:
        if CELL_PADDING is None:
            CELL_PADDING = 100

        external_multiplier = 10000
        external_offset = np.pi + np.pi*rotation/180
    else:
        if CELL_PADDING is None:
            CELL_PADDING = 40

        external_multiplier = 1
        external_offset = 0 + np.pi*rotation/180
    
    r = render_rna.RNARenderer()

    pairmap = render_rna.get_pairmap_from_secstruct(secstruct)
    pairs = []
    for i in range(len(pairmap)):
        if pairmap[i] > i:
            pairs.append({"from":i, "to":pairmap[i], "p":1.0, "color":COLORS["e"]})
    
    pk_pairmap = render_rna.get_pk_pairmap_from_secstruct(secstruct)
    pk_pairs = []
    for i in range(len(pk_pairmap)):
        if pk_pairmap[i] > i:
            pk_pairs.append({"from":i, "to":pk_pairmap[i], "p":1.0, "color":COLORS["e"]})
    

    r.setup_tree(secstruct, NODE_R, PRIMARY_SPACE, PAIR_SPACE, external_multiplier, external_offset, flipped)

    # if colors are numeric, create color scale

    do_show_colorbar = True if show_colorbar is True else False

    if ext_color_file:
        if show_colorbar == 'auto':
            do_show_colorbar = True

        if vmin is None:
            vmin=np.min(color_list)
        if vmax is None:
            vmax=np.max(color_list)

        colormap = plt.get_cmap(cmap_name) 
        cNorm  = mcolors.Normalize(vmin=vmin, vmax=vmax)
        scalarMap = cm.ScalarMappable(norm=cNorm, cmap=colormap)
        colors = [scalarMap.to_rgba(val)[:-1] for val in color_list]
        colors = [[x*256 for x in y] for y in colors]

    else:
        if isinstance(color_list[0],str) and color_list[0].isalpha():
            colors = [COLORS[x] for x in list(color_list)]

        else: #if isinstance(color_list[0],float):
            print('Interpreting color string as integer values')
            if show_colorbar == 'auto':
                do_show_colorbar = True
            colors = [float(x) for x in color_list]
            colormap = plt.get_cmap(cmap_name) 
            if vmin is None:
                vmin=np.min(colors)
            if vmax is None:
                vmax=np.max(colors)

            cNorm  = mcolors.Normalize(vmin=vmin, vmax=vmax)
            scalarMap = cm.ScalarMappable(norm=cNorm, cmap=colormap)
            colors = [scalarMap.to_rgba(val)[:-1] for val in colors]
            colors = [[x*256 for x in y] for y in colors]

    size = r.get_size()
    if do_show_colorbar:
        size[1] += size[0] / 4 / 4 * 1.25

    if large_mode or movie_mode:
        r.xarray_ = [x - r.xarray_[0] for x in r.xarray_]
        r.yarray_ = [y - r.yarray_[0] for y in r.yarray_]
        #print(np.min(r.xarray_), np.max(r.xarray_))
        #print(np.min(r.yarray_), np.max(r.yarray_))

        cell_size_x = np.max(r.xarray_) - np.min(r.xarray_) + CELL_PADDING * 2
        cell_size_y = np.max(r.yarray_) - np.min(r.yarray_) + CELL_PADDING * 2

        if do_show_colorbar:
            cell_size_y += cell_size_x / 4 / 4 * 1.25

        #print(cell_size_x, cell_size_y)

    else:
        cell_size_x = (max(size) if square else size[0]) + CELL_PADDING * 2
        cell_size_y = (max(size) if square else size[1]) + CELL_PADDING * 2

    if svg_mode:
        # drawing object writes an svg file
        drawing_obj = svg.svg("%s.svg" % filename, cell_size_x, cell_size_y, CELL_PADDING)
    else:
        if ax is None:
            fig, ax = plt.subplots(1,1,figsize=(cell_size_x/72, cell_size_y/72), layout='constrained')
            drawing_obj = mpl.mpl(ax=ax)

        else:
            drawing_obj = mpl.mpl(ax=ax)

    if movie_mode or large_mode:
        r.draw(drawing_obj, CELL_PADDING, cell_size_y-CELL_PADDING,
         colors, pairs if draw_pairs else None, pk_pairs if draw_pseudoknots else None, sequence, RENDER_IN_LETTERS, external_offset, flipped, line, svg_mode, alpha, direction_labels)
    else:
        r.draw(drawing_obj, CELL_PADDING, CELL_PADDING, colors,
         pairs if draw_pairs else None, pk_pairs if draw_pseudoknots else None, sequence, RENDER_IN_LETTERS, external_offset, flipped, line, svg_mode, alpha, direction_labels)

    if do_show_colorbar:
        drawing_obj.colorbar(cmap_name=cmap_name, label=colorbar_label, aspect=4, fraction=0.05, orientation="horizontal")

    if not svg_mode:
        # apply matplotlib settings
        drawing_obj.clean_up()

def parse_colors(color_string):
    colorings = color_string.strip().split(",")
    colors = []
    for coloring in colorings:
        if "x" in coloring:
            [n, color] = coloring.split("x")
            colors += int(n) * [color]
        else:
            colors += [coloring]
    return colors[0]

def reorder_strands(order, seq, colors):
    breaks = [-1] + [m.start() for m in re.finditer("&", seq)] + [len(seq)]
    seq_segments = seq.split("&")
    color_segments = [colors[breaks[i]+1:breaks[i+1]] for i in range(len(breaks)-1)]

    seq = ""
    colors = []
    for strand in order:
        seq += seq_segments[strand-1] + "&"
        colors += color_segments[strand-1] + ["w"]
    return [seq[:-1], colors[:-1]]

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("filename", help="filename specifying sequences to fold", type=str)
    parser.add_argument("-f", "--fold", help="automatic folding", type=str)
    args = parser.parse_args()

    with open(args.filename) as f:
        n = int(f.readline())
        for i in range(n):
            seq = f.readline().strip()
            if args.fold == "nupack":
                result = inv_utils.nupack_fold(seq, 1e-7)
                secstruct = result[0]
                colors = parse_colors(f.readline())
                seq, colors = reorder_strands(result[2], seq, colors)
            elif args.fold == "vienna":
                secstruct = inv_utils.vienna_fold(seq)[0]
                colors = parse_colors(f.readline())
            else:
                secstruct = f.readline().strip()
                colors = parse_colors(f.readline())
            seq.replace("&", "")
            secstruct.replace("&", "")
            print(colors)
            draw_rna(seq, secstruct, colors, "%s_%d" % (args.filename, i))

if __name__ == "__main__":
    main()



import os
import random, string
import re
from subprocess import Popen, PIPE, STDOUT, check_call

DEFAULT_TEMPERATURE = 37.0
DEFAULT_PARAMS = 'rna'
BASES = ['A','U','G','C']
BIN_DIR = "bin"
#fold a sequence
#@param seq:sequence
#@return: [parenthesis notation, energy]

def vienna_fold(seq, cotransc=False, constraint=False):
    """
    folds sequence using Vienna

    args:
    seq is the sequence string

    returns:
    secondary structure
    """
    # run ViennaRNA
    if constraint:
        options = "-C"
        input = seq + "\n" + constraint
    else:
        options = ""
        input = ''.join(seq)
    if cotransc:
        p = Popen([os.path.join(BIN_DIR,'CoFold'), '--distAlpha', '0.5', '--distTau', '640', '--noPS', options], stdout=PIPE, stdin=PIPE, stderr=STDOUT)
    elif '&' in seq:
        p = Popen([os.path.join(BIN_DIR,'RNAcofold'), '-T','37.0', '-noPS', options], stdout=PIPE, stdin=PIPE, stderr=STDOUT)
    else:
        p = Popen([os.path.join(BIN_DIR,'RNAfold'), '-T','37.0', '-noPS', options], stdout=PIPE, stdin=PIPE, stderr=STDOUT)
    pair= p.communicate(input=input)[0]
    p.wait()

    # split result by whitespace
    toks = re.split(r'\s+| \(?\s?',pair)
    ret= []
    ret.append(toks[1])
    ret.append(toks[2][1:-1])
    return ret

def nupack_fold(seq, oligo_conc, bpp = False):
    """
    folds sequence using nupack
    """
    os.system("source ~/.bashrc")
    rand_string = ''.join(random.choice(string.ascii_lowercase) for _ in range(5))
    split = seq.split("&")
    with open("%s.in" % rand_string, "w") as f:
        f.write("%s\n" % len(split))
        for seq in split:
            f.write("%s\n" % seq)
        f.write("1\n")
    os.system("cp %s/%s.list %s.list" % (BIN_DIR, len(split), rand_string))
    with open("%s.con" % rand_string, "w") as f:
        f.write("%s\n" % oligo_conc * (len(split)-1))
        f.write("1e-9\n")
    options = ['-material', DEFAULT_PARAMS, '-ordered', '-mfe']#, '-quiet']
    if bpp:
        options.append('-pairs')
    p = Popen([os.path.join(BIN_DIR,'complexes')] + options + [rand_string], stdout=PIPE, stdin=PIPE, stderr=STDOUT)
    p.wait()
    p = Popen([os.path.join(BIN_DIR,'concentrations'), '-ordered', rand_string], stdout=PIPE, stdin=PIPE, stderr=STDOUT)
    p.wait()
    # get mfe
    with open("%s.eq" % rand_string) as f_eq:
        for line in f_eq:
            if not line.startswith("%"):
                mfe = line.strip().split()
                if int(mfe[-3]):
                    break
    # get strand ordering
    with open("%s.ocx-key" % rand_string) as f_key:
        for line in f_key:
            if line.startswith("%s\t%s" % (mfe[0], mfe[1])):
                strands = [int(x) for x in line.strip().split()[2:]]
                break
    # get mfe structure
    with open("%s.ocx-mfe" % rand_string) as f_mfe:
        line = f_mfe.readline()
        while line:
            if line.startswith("%% complex%s-order%s" % (mfe[0], mfe[1])):
                f_mfe.readline()
                energy = f_mfe.readline().strip()
                secstruct = f_mfe.readline().strip()
                break
            line = f_mfe.readline()

    if bpp:
        bpp_matrix = []
        with open("%s.cx-epairs" % rand_string) as f_pairs:
            line = f_pairs.readline()
            while line:
                if line.startswith("%% complex%s" % complex[0]):
                    f_pairs.readline()
                    line = f_pairs.readline()
                    while not line.startswith("%"):
                        bpp_matrix.append(line.strip().split())
                line = f_mfe.readline()
        os.system("rm %s*" % rand_string)
        return bpp_matrix

    # get full secondary structure
    for i in range(len(split)):
        if i+1 not in strands:
            secstruct += "&" + "."*len(split[i])
            strands.append(i+1)
    os.system("rm %s*" % rand_string)
    return [secstruct.replace("+", "&"), energy, strands]




import matplotlib.pyplot as plt
import draw_rna.draw as d
from draw_rna.draw_utils import seq2col

def draw_struct(seq, secstruct, c=None, line=False, large_mode=False,
 cmap='viridis', rotation=0, vmin=None, vmax=None, alpha=None, ax=None, flipped=False):
    '''
    Draw sequence with secondary structure.
    Inputs:
    c (string or array-like).  If string, characters must correspond to colors.
     If array-like obj, used as mapping for colormap (provided in cmap), or a string.
    line (bool): draw secstruct as line.
    large_mode: draw outer loop as straight line.
    rotation: rotate molecule (in degrees).
    '''

    if c is not None:
        assert len(c) == len(seq)
        if isinstance(c[0], float):
            d.draw_rna(seq, secstruct, c, line=line, ext_color_file=True, cmap_name = cmap, vmin=vmin, vmax=vmax,
             rotation=rotation, large_mode = large_mode, alpha=alpha, ax=ax, flipped=flipped)
        else:
            d.draw_rna(seq, secstruct, c,  line=line, cmap_name=cmap, large_mode=large_mode, vmin=vmin, vmax=vmax,
             rotation=rotation, alpha=alpha, ax=ax, flipped=flipped)

    else:
        d.draw_rna(seq, secstruct, seq2col(seq), line=line, cmap_name = cmap, vmin=vmin, vmax=vmax,
         large_mode = large_mode, rotation=rotation, alpha=alpha, ax=ax, flipped=flipped)

    if ax is None:
        plt.show()

    #return plt.gca()


import draw_rna.draw as d
from draw_rna.draw_utils import seq2col

import argparse, sys, os
import numpy as np

def main():
    p = argparse.ArgumentParser()
    p.add_argument('inputfile')
    p.add_argument('--png', action='store_true')
    p.add_argument('--movie_mode', action='store_true')
    p.add_argument('--large_mode',action='store_true')
    p.add_argument('--line', action='store_true')
    p.add_argument('--color_values', nargs='*', action='store',
     dest='color_value_file', help='File containing values (i.e. reactivities)')
    p.add_argument('--colormap', action='store',default= 'viridis')
    p.add_argument('--flipped', action='store_true')
    args = p.parse_args()

    ext_color_file=False

    with open(args.inputfile) as f:
        for line in f:
            if not line.startswith('#') and line.strip() != '':
                try:
                    name = line.strip().replace(' ', '_')
                    seq = next(f).strip()
                    secstruct = next(f).strip()
                except Exception as e:
                    print(e)
                    raise ValueError('improper format')
                try:
                    color_string = next(f).strip()
                    if len(color_string)<1:
                        color_string = None
                except:
                    color_string=None
        
                print('drawing %s' % name)

                if args.color_value_file is not None:
                    print('Reading in colors from %s' % args.color_value_file[0])
                    color_vec = np.loadtxt(args.color_value_file[0])
                    ext_color_file=True
                    assert(len(color_vec) == len(seq))

                if ext_color_file:
                    d.draw_rna(seq, secstruct, color_vec, name, line=args.line, ext_color_file=True, 
                        cmap_name=args.colormap, large_mode=args.large_mode,
                        movie_mode=args.movie_mode, svg_mode=True, flipped=args.flipped)
                elif color_string:
                    d.draw_rna(seq, secstruct, color_string, name, line=args.line, cmap_name = args.colormap,
                     large_mode = args.large_mode, movie_mode=args.movie_mode, svg_mode=True, flipped=args.flipped)
                else:
                    d.draw_rna(seq, secstruct, seq2col(seq), name, line=args.line, cmap_name = args.colormap,
                     large_mode = args.large_mode, movie_mode=args.movie_mode, svg_mode=True, flipped=args.flipped)
                
                if args.png:
                    if 'INKSCAPEDIR' not in os.environ:
                        print('Please set INKSCAPEDIR environmental variable with path to Inkscape app.')
                        return
                    inkscape = '%s/Contents/Resources/bin/inkscape' % os.environ['INKSCAPEDIR']
                    if not os.path.isfile(inkscape):
                        print('Inkscape not found. Please update INKSCAPEDIR environmental variable with path to Inkscape app.')
                        return
                    os.system('%s --export-png $(pwd)/%s.png --export-background-opacity 1 $(pwd)/%s.svg' % (inkscape, name, name))
                    os.system('rm $(pwd)/%s.svg' % (name))

if __name__ == '__main__':
    main()



"""Methods for outputting SVG files."""
from io import BytesIO
from base64 import b64encode
import numpy as np
from matplotlib import pyplot as plt
from matplotlib.colors import Normalize
from matplotlib.cm import ScalarMappable
from draw_rna.draw_utils import *

class svg(object):
	def __init__(self, filename, w, h, padding):
		self.w = w
		self.h = h
		self.padding = padding

		self.miny = 0
		self.maxy = h

		self.__out = open(filename, 'w')

		self.elements = []

	def __del__(self):
		# write the header
		self.__out.write("""
		<?xml version="1.0" encoding="utf-8"?>
		  <svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg"
		  x="0px" y="0px"
		  width="%spx" height="%spx"
		  viewBox="%s %s %s %s"
		  enable-background="new 0 0 247.44 234.492"
		  xml:space="preserve">""".replace('\t', '')[1:] % (self.w, self.maxy - self.miny, 0, self.miny, self.w, self.maxy - self.miny))
		self.__out.write('\n')

		for element in self.elements:
			self.__out.write(element)
		
		# write a footer
		self.__out.write('</svg>\n')

		self.__out.close()
		del self.__out

	def line(self, x1, y1, x2, y2, stroke, width=1, alpha=1):
		""""""
		# print 'Line (%s %s %s %s %s)' % (x1, y1, x2, y2, color)
		stroke = convert_color(stroke)
		self.elements.append('<line fill="none" stroke="%s" stroke-width="%dpx" x1="%s" y1="%s" x2="%s" y2="%s" x3="0.0" y3="0.0" alpha="%s"/>\n' %
			(stroke, width, x1, y1, x2, y2,alpha))

	def polygon(self, points, fill, stroke, opacity = 1.0):
		fill = convert_color(fill)
		stroke = convert_color(stroke)
		points = ' '.join('%s,%s' % (x,y) for (x,y) in points)
		self.elements.append('<polygon fill="%s" stroke="%s" points="%s" opacity="%f"/>\n' %
			(fill, stroke, points,opacity))

	def circle(self, x, y, radius, fill, stroke, alpha=1):
		fill = convert_color(fill)
		self.elements.append('<circle cx="%s" cy="%s" r="%s" fill="%s" stroke="%s" alpha="%s"/>\n' %
			(x, y, radius, fill, stroke,alpha))

	def text(self, x, y, size, fill, align, string,alpha=1,align_y='center'):
		if y < self.miny:
			self.miny = y - self.padding

		fill = convert_color(fill)
		if align_y == 'center':
			dominant_baseline = 'middle'
		elif align_y == 'top':
			dominant_baseline = 'hanging'
		self.elements.append(' <text x="%d" y="%d" font-family="sans_serif" font-size="%d" fill="%s" text-anchor="%s" dominant-baseline="%s" alpha="%s">%s</text>' % (x,y,size,fill,'middle',dominant_baseline,alpha,string))
        ## rotated
		#self.elements.append(' <text x="%d" y="%d" font-family="sans_serif" font-size="%d" fill="%s" text-anchor="%s" transform="rotate(180 %d,%d)">%s</text>' % (x-10,y+10,size,fill,align,x,y,str))

	def colorbar(self, cmap_name, fraction, aspect, label=None, **kwargs):
		h = self.h * fraction
		w = h * aspect

		fig, ax = plt.subplots(figsize=(w / 72, h / 72))
		fig.subplots_adjust(left=0, right=1, top=1, bottom=0)
		fig.colorbar(ScalarMappable(norm=Normalize(0, 1), cmap=cmap_name), cax=ax, **kwargs)
		
		out = BytesIO()
		fig.savefig(out, bbox_inches='tight', pad_inches=0, transparent=True, dpi=300)
		out.seek(0)

		bbox = fig.get_tightbbox()
		final_width = bbox.width * 72
		final_height = bbox.height * 72

		x = (self.w  - final_width) / 2
		y = self.h
		self.elements.append('<image href="data:image/png;base64,%s" x="%s" y="%s" width="%s" height="%s" />' % (b64encode(out.read()).decode(), x, y, final_width, final_height))

		if label:
			self.text(self.w/2, y + final_height + final_height/16, final_height // 2, "#000000", 'center', label, align_y='top')
			self.maxy = y + final_height + final_height/16 + final_height//2 + self.padding
		else:
			self.maxy = y + final_height + self.padding



def clamp(x): 
  return int(max(0, min(x, 255)))

def convert_color(color):
	"""Converts a color object (be it touple-like, or string to an SVG-readable color string)."""
	if type(color) == str:
		return color

	else:
		r,g,b = color
		return "#{0:02x}{1:02x}{2:02x}".format(clamp(r), clamp(g), clamp(b))

def seq2col(seq):
    color = {'A': 'y', 'U':'b', 'G':'r', 'C':'g', 'T': 'b', 'N': 'k', ' ': 'w'}
    col = []
    for c in seq:
        col.append(color[c])
    return col


import matplotlib.pyplot as plt
from matplotlib.colors import Normalize
from matplotlib.cm import ScalarMappable
from draw_rna.draw_utils import * 

class mpl(object):
	def __init__(self, ax, dpi=72):
		# create the file
		#self.dpi = dpi
		#plt.figure(figsize=(w/self.dpi,h/self.dpi), dpi=self.dpi)
		self.ax=ax
		plt.sca(self.ax)

	def line(self, x1, y1, x2, y2, stroke, width=1, alpha=1):
		""""""
		# print 'Line (%s %s %s %s %s)' % (x1, y1, x2, y2, color)
		stroke = convert_color(stroke)

		plt.plot([x1,x2], [y1,y2], linewidth=width, c=stroke, zorder=0, alpha=alpha)

	def circle(self, x, y, radius, fill, stroke, alpha=1):
		fill = convert_color(fill)
		plt.scatter(x,y,s=1,zorder=0,alpha=alpha)
		circ = plt.Circle((x,y),radius=radius, alpha=alpha, color=fill, zorder=1, linewidth=0)
		self.ax.add_artist(circ)
	
	def text(self, x, y, size, fill, align, string, alpha=1, align_y='center'):
		fill = convert_color(fill)
		plt.text(x,y,string, fontsize=size, color=fill, zorder=2, horizontalalignment=align, verticalalignment=align_y, alpha=alpha)
        ## rotated 
		#self.__out.write(' <text x="%d" y="%d" font-family="sans_serif" font-size="%d" fill="%s" text-anchor="%s" transform="rotate(180 %d,%d)">%s</text>' % (x-10,y+10,size,fill,align,x,y,str))
	
	def colorbar(self, cmap_name, **kwargs):
		plt.colorbar(ScalarMappable(norm=Normalize(0, 1), cmap=cmap_name), ax=self.ax, **kwargs)

	def clean_up(self):
		self.ax.set_aspect('equal')
		plt.axis('off')
		l, r = plt.xlim()
		plt.xlim(l-20,r+20)
		b,t = plt.ylim()
		plt.ylim(b-20,t+20)
